/*
 * Software License Agreement (BSD License)
 *
 *  Point Cloud Library (PCL) - www.pointclouds.org
 *  Copyright (c) 2010-2011, Willow Garage, Inc.
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of Willow Garage, Inc. nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

#pragma once

#include <pcl/common/common.h>
#include <pcl/memory.h>
#include <pcl/pcl_macros.h> // for PCL_EXPORTS

namespace pcl {
template <class FeatureType,
          class DataSet,
          class LabelType,
          class ExampleIndex,
          class NodeType>
class PCL_EXPORTS DecisionTreeTrainerDataProvider {

  /** The training data set. */
  DataSet data_set_;
  /** The label data. */
  std::vector<LabelType> label_data_;

public:
  using Ptr = shared_ptr<DecisionTreeTrainerDataProvider<FeatureType,
                                                         DataSet,
                                                         LabelType,
                                                         ExampleIndex,
                                                         NodeType>>;
  using ConstPtr = shared_ptr<const DecisionTreeTrainerDataProvider<FeatureType,
                                                                    DataSet,
                                                                    LabelType,
                                                                    ExampleIndex,
                                                                    NodeType>>;

  /** Constructor. */
  DecisionTreeTrainerDataProvider() = default;

  /** Destructor. */
  virtual ~DecisionTreeTrainerDataProvider() = default;

  /** Virtual function called to obtain training examples and labels before
   *  training a specific tree */
  virtual void
  getDatasetAndLabels(DataSet& data_set,
                      std::vector<LabelType>& label_data,
                      std::vector<ExampleIndex>& examples) = 0;
};

} // namespace pcl
